==> appslandia-common-4.2-sources.jar: com/appslandia/common/threading/ConcurrentService.java <==
// The MIT License (MIT)
// Copyright © 2015 AppsLandia. All rights reserved.

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

package com.appslandia.common.threading;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import com.appslandia.common.base.InitializeObject;
import com.appslandia.common.utils.AssertUtils;

/**
 *
 * @author <a href="mailto:haducloc13@gmail.com">Loc Ha</a>
 *
 */
public abstract class ConcurrentService extends InitializeObject {

	protected final Map<Object, Object> barriers = new HashMap<>();
	protected final Object defaultMutex = new Object();

	public abstract ExecutorService getExecutorService();

	protected Object getMutex(Object barrierId) {
		Object mutex = this.barriers.get(barrierId);
		return (mutex != null) ? mutex : this.defaultMutex;
	}

	public <V> V execute(Object barrierId, Callable<V> task) throws Exception {
		return execute(barrierId, task, 0, null);
	}

	public <V> V execute(final Object barrierId, final Callable<V> task, long timeout, TimeUnit unit) throws Exception {
		this.initialize();
		AssertUtils.assertNotNull(barrierId);

		Future<V> result = this.getExecutorService().submit(new Callable<V>() {

			@Override
			public V call() throws Exception {
				synchronized (getMutex(barrierId)) {
					return task.call();
				}
			}
		});
		if (timeout <= 0) {
			return result.get();
		} else {
			return result.get(timeout, unit);
		}
	}

	public ConcurrentService addBarrier(Object barrierId) {
		this.assertNotInitialized();
		AssertUtils.assertNotNull(barrierId);
		this.barriers.put(barrierId, new Object());
		return this;
	}
}

==> appslandia-common-7.2-sources.jar: com/appslandia/common/threading/ConcurrentService.java <==
// The MIT License (MIT)
// Copyright © 2015 AppsLandia. All rights reserved.

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

package com.appslandia.common.threading;

import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;

import com.appslandia.common.utils.AssertUtils;

/**
 *
 * @author <a href="mailto:haducloc13@gmail.com">Loc Ha</a>
 *
 */
public abstract class ConcurrentService<K> extends MutexService<K> {

	public abstract ExecutorService getExecutorService();

	public <V> Future<V> execute(final K mutexKey, final Callable<V> task) throws Exception {
		this.initialize();
		AssertUtils.assertNotNull(mutexKey);

		return this.getExecutorService().submit(new Callable<V>() {

			@Override
			public V call() throws Exception {
				synchronized (getMutex(mutexKey)) {
					return task.call();
				}
			}
		});
	}
}
